//
//  ViewController.swift
//  IR
//
//  Created by Phidgets on 2018-02-02.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = IR()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    var repeatCount = 0
    
    @IBOutlet var codeStack: UIStackView!
    @IBOutlet var learnStack: UIStackView!
    @IBOutlet var rawStack: UIStackView!
    
    @IBOutlet var codeLabel: UILabel!
    @IBOutlet var bitCountLabel: UILabel!
    @IBOutlet var repeatCountLabel: UILabel!
    
    @IBOutlet var learnCodeLabel: UILabel!
    @IBOutlet var encodingLabel: UILabel!
    @IBOutlet var lengthLabel: UILabel!
    @IBOutlet var gapLabel: UILabel!
    @IBOutlet var trailLabel: UILabel!
    @IBOutlet var headerLabel: UILabel!
    @IBOutlet var oneLabel: UILabel!
    @IBOutlet var zeroLabel: UILabel!
    @IBOutlet var repeatLabel: UILabel!
    @IBOutlet var toggleMaskLabel: UILabel!
    @IBOutlet var rawDataText: UITextView!
    
    @IBOutlet var allLabels: [UILabel]!
    @IBOutlet var topConstraint: NSLayoutConstraint!
    
    var RAWDATA_LONGSPACE = 4294967295
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.code.addHandler(code_handler)
            let _ = ch.learn.addHandler(learn_handler)
            let _ = ch.rawData.addHandler(raw_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
 
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            DispatchQueue.main.async {
                self.codeStack.isHidden = false
                self.learnStack.isHidden = false
                self.clearAll()
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.codeStack.isHidden = true
                self.learnStack.isHidden = true
                self.clearAll()
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func code_handler(sender: IR, data: (code: String, bitCount: UInt32, isRepeat: Bool)){
        DispatchQueue.main.async {
            self.codeLabel.text = "0x" + data.code
            self.bitCountLabel.text = String(format:"%d", data.bitCount)
            self.repeatCount += 1
            if(!data.isRepeat){
                self.repeatCount = 0
            }
            self.repeatCountLabel.text = String(format:"%d", self.repeatCount)
        }
    }
    
    func learn_handler(sender: IR, data: (code: String, codeInfo: IRCodeInfo)){
        DispatchQueue.main.async {
            var tempString = ""
            self.learnCodeLabel.text = "0x" + data.code
            self.encodingLabel.text = (data.codeInfo.encoding == IRCodeEncoding.biPhase) ? "biPhase" : (data.codeInfo.encoding == IRCodeEncoding.pulse) ? "pulse" : (data.codeInfo.encoding == IRCodeEncoding.RC5) ? "RC5" : (data.codeInfo.encoding == IRCodeEncoding.RC6) ? "RC6" : (data.codeInfo.encoding == IRCodeEncoding.space) ? "space" : "unknown"
            self.lengthLabel.text = (data.codeInfo.length == IRCodeLength.constant) ? "constant" : (data.codeInfo.length == IRCodeLength.variable) ? "variable" : "unknown"
            self.gapLabel.text = String(format:"%d",data.codeInfo.gap)
            self.trailLabel.text = String(format:"%d", data.codeInfo.trail)
            
            for val in data.codeInfo.header{
                tempString = tempString + String(format:"%d,", val)
            }
            tempString.removeLast()
            self.headerLabel.text = tempString
            tempString = ""
            
            for val in data.codeInfo.one{
                tempString = tempString + String(format:"%d,", val)
            }
            tempString.removeLast()
            self.oneLabel.text = tempString
            tempString = ""
            
            for val in data.codeInfo.zero{
                tempString = tempString + String(format:"%d,", val)
            }
            tempString.removeLast()
            self.zeroLabel.text = tempString
            tempString = ""
            
            for val in data.codeInfo.repeatCode{
                if(val != 0){
                    tempString = tempString + String(format:"%d,", val)
                }
            }
            tempString.removeLast()
            self.repeatLabel.text = tempString
            tempString = ""
            
            self.toggleMaskLabel.text = (data.codeInfo.toggleMask == "") ? "No Toggle Mask" : "0x" + data.codeInfo.toggleMask
        }
    }
    
    func raw_handler(sender: IR, data: [UInt32]){
        DispatchQueue.main.async {
            var rawString = ""
            if(!self.rawStack.isHidden){
                for value in data{
                    if(value == self.RAWDATA_LONGSPACE){
                        rawString = rawString + "\n\nLONG\n"
                    }
                    else if(value > 100000){
                        rawString = rawString + String(format:"\n\n%d\n", value)
                    }
                    else{
                        rawString = rawString + String(format:"%d,", value)
                    }
                }
                self.rawDataText.text = self.rawDataText.text + rawString
            }
            self.rawDataText.scrollRangeToVisible(NSMakeRange(0, 1))
        }
    }
    
    //MARK: GUI controls
    @IBAction func retransmit(_ sender: Any) {
        do{
            let lastCode = try ch.getLastLearnedCode()
            try ch.transmit(code: lastCode.code, codeInfo: lastCode.codeInfo)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func showRawData(_ sender: Any) {
        DispatchQueue.main.async {
            self.learnStack.isHidden = true
            self.rawStack.isHidden = false
            self.topConstraint.constant = 10
        }
    }
    
    @IBAction func clear(_ sender: Any) {
        DispatchQueue.main.async {
            self.rawDataText.text = ""
        }
    }
    
    @IBAction func hide(_ sender: Any) {
        DispatchQueue.main.async {
            self.rawStack.isHidden = true
            self.learnStack.isHidden = false
            self.topConstraint.constant = 320
        }
    }
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func clearAll(){
        DispatchQueue.main.async {
            for label in self.allLabels{
                label.text = ""
            }
        }
    }
}

